# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

class Parameter(object):
    name = None
    type = None
    description = None
    optional = None

    def __init__(self, name, type, description = None, optional = False):
        self.name = name
        self.type = type
        self.description = description
        self.optional = optional
    
    def __str__(self):
        if self.optional:
            return '[%s]' % self.name
        else:
            return '<%s>' % self.name

class Command(object):
    name = None
    parameters = None
    description = None
    commandHandler = None
    rights = None
    
    def __init__(self, name, parameters, description, commandHandler, rights = 0):
        self.name = name
        self.parameters = parameters
        self.description = description
        self.commandHandler = commandHandler
        self.rights = rights
        
    def requiredParameters(self):
        return len([parameter for parameter in self.parameters if not parameter.optional])
        
    def handle(self, connection, parameters):
        if connection.rights < self.rights:
            return 'Not enough rights'
        if len(parameters) < self.requiredParameters():
            return 'Not enough parameters'
        if len(parameters) > len(self.parameters):
            return 'Invalid number of arguments specified'
        parameterDict = {}
        for i in range(len(parameters)):
            parameter = self.parameters[i]
            if parameter.type == bool:
                type = int
            else:
                type = parameter.type
            parameterDict[parameter.name] = type(parameters[i])
        return self.commandHandler(connection, **parameterDict)
        
    def getDescription(self):
        prepare = [self.__str__()]
        for parameter in self.parameters:
            prepare.append('%s %s: %s' % (parameter.type.__name__, parameter.name,
                parameter.description or ''))
        return '\n'.join(prepare)
    
    def __str__(self):
        return '!%s %s' % (self.name, 
            ' '.join([str(parameter) for parameter in self.parameters]))

class Commands(object):
    commands = None
    def __init__(self, commands = []):
        self.commands = commands
        
    def addCommand(self, command):
        self.commands.append(command)
        
    def findCommand(self, commandName):
        try:
            return [command for command in self.commands 
                    if command.name.lower() == commandName.lower()][0]
        except IndexError:
            return False
            
    def getDescription(self):
        '--\n'.join([command.getDescription() for command in self.commands])