# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

import time
import math

from pySndo.locals import *

def getAngle(x1, y1, x2, y2, numberOfAngles = 32):
    deltax = x2 - x1
    deltay = (y2 - y1) * -1
    angle_rad = math.atan2(deltay,deltax)
    angle_deg = (angle_rad*180.0/math.pi + 360) % 360
    return angle_deg / (360.0 / numberOfAngles)

class Movement(object):
    startX = None
    startY = None
    speed = None
    started = None
    direction = 0
    moving = False
    _angle = None

    def set(self, startX, startY, direction, speed = PLAYER_SPEED):
        self.startX = startX
        self.startY = startY
        self.direction = direction
        self._angle = math.radians(direction * 11.25)
        self.speed = speed

    def start(self):
        self.started = time.time()
        self.moving = True
        
    def stop(self):
        self.startX, self.startY = self.getPosition()
        self.moving = False

    def getPosition(self):
        if not self.moving:
            return self.startX, self.startY
        timePassed = time.time()-self.started
        elapsedPixels = self.speed*timePassed
        currentX = self.startX + (elapsedPixels * math.cos(self._angle))
        currentY = self.startY + (elapsedPixels * math.sin(self._angle) * -1)
        return int(currentX), int(currentY)
        
    def finishedMoving(self):
        # implement me, check for obstacles, etc.
        raise NotImplementedError

    def getFinishTime(self):
        # implement me, check for obstacles, etc.
        raise NotImplementedError

class Player(object):
    connection = None
    
    movement = None

    typing = False
    look_direction = 24
    reloading = False
    
    alive = False
    
    team = 0
    
    health = None
    
    kills = 0
    deaths = 0

    def __init__(self, connection = None):
        self.movement = Movement()
        self.connection = connection
        
    def damage(self, damage):
        self.health = max(0, self.health - damage)
        return self.health == 0
        
    def getTeam(self):
        return playerTeam[self.team]
        
    def setTeam(self, teamName):
        self.team = playerTeam.index(teamName)
        
    def directionToPlayer(self, player):
        x, y = self.movement.getPosition()
        playerX, playerY = player.movement.getPosition()
        return getAngle(x, y, playerX, playerY)