# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

from pySndo.map.loader import Map, Data
from pySndo.locals import create_number
from pySndo.exceptions import HackingAttempt
from pySndo.game.weapon import Glock, SIGP, DesertEagle, Benelli, MP5, Galil, \
    SVD, AK47, Colt, G3A3, SR60, AW50

# key used to encrypt\decrypt info data
infoKey = '\xF8\x38\xFA\x32\x4A\xDB\xAE\x91\x3D\xD5\
\x2E\x99\xB3\x5F\x79\x7F\x2F\x55\x25\x30\x43\xD9'

class _Empty(object):
    def __init__(self, data = None):
        if data != None:
            raise Exception('packet contained data on empty packet type')
            
    def generate(self):
        return None

class GetInitial(_Empty):
    pass

class RequestMapLine(object):
    mapName = None
    requestedLine = None
    def __init__(self, data = None):
        if data:
            splitted = data.split(',')
            self.requestedLine = int(splitted.pop(0))
            self.mapName = splitted.pop(0)
    
    def generate(self):
        return ','.join([str(self.requestedLine), self.mapName])

class GetMapSending(_Empty):
    pass

requestLoaders = [
    GetInitial,
    GetMapSending,
    RequestMapLine
]

class ClientRequest(object):
    type = None
    def __init__(self, data = None):
        if data:
            splitted = data.split(',')
            request = int(splitted.pop(0))
            if splitted:
                loaderData = ','.join(splitted)
            else:
                loaderData = None
            self.type = requestLoaders[request](loaderData)
            
    def generate(self):
        request = requestLoaders.index(self.type.__class__)
        itemList = [create_number(request, 3), self.type.generate()]
        return ','.join([item for item in itemList if item != None])

class InitialMessage(object):
    mapName = ''
    mapCrc = 0
    welcome = ''
    serverVersion = '1.44'
    currentPlayers = 0
    maxPlayers = 0
    
    weapons = {
        DesertEagle : True,
        Benelli : True,
        MP5 : True,
        AK47 : True,
        SVD : True,
        Colt : True,
        G3A3 : True,
        AW50 : True,
        Galil : True,
        SR60 : True
    }
    
    password = None
    
    gameKey = None
    
    def __init__(self, data = None):
        weapons = self.weapons.copy()
        self.weapons = weapons
        if data:
            splitted = data.split(',')
            self.mapName = splitted.pop(0)
            self.mapCrc = int(splitted.pop(0))
            welcome = splitted.pop(0)
            if welcome == ' ':
                welcome = None
            self.welcome = welcome
            
            weapons[DesertEagle] = bool(int(splitted.pop(0)))
            weapons[Benelli] = bool(int(splitted.pop(0)))
            weapons[MP5] = bool(int(splitted.pop(0)))
            weapons[AK47] = bool(int(splitted.pop(0)))
            weapons[SVD] = bool(int(splitted.pop(0)))
            weapons[Colt] = bool(int(splitted.pop(0)))
            weapons[G3A3] = bool(int(splitted.pop(0))) 
            weapons[AW50] = bool(int(splitted.pop(0)))
            weapons[Galil] = bool(int(splitted.pop(0)))
            weapons[SR60] = bool(int(splitted.pop(0)))
            
            self.serverVersion = splitted.pop(0)
            
            self.currentPlayers = int(splitted.pop(0))
            self.maxPlayers = int(splitted.pop(0))
            
            password = splitted.pop(0)
            if password == ' ':
                password = None
            self.password = password
            
            self.gameKey = ','.join(splitted)
        
    def loadFromMap(self, map):
        self.mapCrc = map.computeChecksum()
        self.mapName = map.name
        
    def generate(self):
        weapons = self.weapons
        return ','.join([str(item) for item in [
            self.mapName, self.mapCrc, self.welcome or ' ',
            int(weapons[DesertEagle]), 
            int(weapons[Benelli]),
            int(weapons[MP5]), 
            int(weapons[AK47]),
            int(weapons[SVD]),
            int(weapons[Colt]),
            int(weapons[G3A3]),
            int(weapons[AW50]),
            int(weapons[Galil]),
            int(weapons[SR60]),
            self.serverVersion, self.currentPlayers,
            self.maxPlayers, self.password or ' ',
            self.gameKey
        ]
        ])


class Error(_Empty):
    pass

class MapLine(object):
    value = None
    def __init__(self, data = None):
        if data:
            self.value = data
            
    def generate(self):
        return self.value

class MapSending(object):
    enabled = None
    def __init__(self, data = None):
        if data:
            if data == 'y':
                self.enabled = True
            elif data == 'n':
                self.enabled = False
        
    def generate(self):
        if self.enabled == True:
            return 'y'
        elif self.enabled == False:
            return 'n'

responseLoaders = [
    None,
    InitialMessage,
    Error,
    MapLine
]

class ServerResponse(object):
    type = None
    def __init__(self, data = None):
        if data:
            splitted = data.split(',')
            try:
                response = int(splitted[0])
                type = responseLoaders[response]
                splitted.pop(0)
            except ValueError:
                type = MapSending
            if splitted:
                loaderData = ','.join(splitted)
            else:
                loaderData = None
            self.type = type(loaderData)
            
    def generate(self):
        try:
            response = create_number(responseLoaders.index(self.type.__class__), 1)
        except ValueError:
            response = None
        itemList = [response, self.type.generate()]
        return ','.join([item for item in itemList if item != None])
            
class MapReceiver(object):
    lineCount = None
    lines = None
    mapName = None
    def __init__(self, mapName = None):
        self.lines = []
        self.mapName = mapName
        
    def feed(self, data):
        if not self.lineCount:
            self.lineCount = int(data)
            return False
        else:
            self.lines.append(data)
            if self.lineCount == len(self.lines):
                return True
            else:
                return False

    def getProgress(self):
        return len(self.lines), self.lineCount
                
    def getNext(self):
        if not self.lineCount:
            return 0
        else:
            return len(self.lines) + 1
                
    def getMap(self):
        mapLines = []
        mapLines.extend(self.lines)
        mapLines.append('')
        data = Data(mapLines)
        map = Map()
        map.name = self.mapName
        map.data = data
        return map
        
class MapSender(object):
    mapLines = None
    _currentLine = 0
    def __init__(self, map):
        dataLines = map.data.generate()[:-1]
        mapLines = [str(len(dataLines))]
        mapLines.extend(dataLines)
        self.mapLines = mapLines
        
    def getLine(self, lineNumber):
        if lineNumber != self._currentLine or lineNumber > len(self.mapLines):
            # hacking attempt
            raise HackingAttempt('illegal map request')
        self._currentLine += 1
        return (self.mapLines[lineNumber], self._currentLine == len(self.mapLines))
    
    def getProgress(self):
        return self._currentLine, len(self.mapLines)