# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

import time

from pySndo.locals import checkDefault

from pySndo.network.site import urlbase64

class Login(object):
    localIP = None
    ircServer = None
    ircPort = None
    ircChannel = None
    welcomeMessage = None
    version = None
    toDo1 = None
    toDo2 = None
    toDo3 = None
    toDo4 = None

    def __init__(self, data = None):
        if data:
            splitted = urlbase64.decode(data).split(',')
            
            checkDefault(splitted.pop(), 'end')
            
            self.localIP = splitted.pop(0)
            self.ircServer = splitted.pop(0)
            self.ircPort = int(splitted.pop(0))
            self.welcomeMessage = splitted.pop(0)
            self.version = splitted.pop(0)
            
            self.toDo1 = int(splitted.pop(0))
            checkDefault(self.toDo1, 0)
            
            self.toDo2 = int(splitted.pop(0))
            checkDefault(self.toDo2, 0)
            
            self.toDo3 = int(splitted.pop(0))
            checkDefault(self.toDo3, 0)
            
            self.toDo4 = int(splitted.pop(0))
            checkDefault(self.toDo4, 0)
            
            self.ircChannel = splitted.pop(0)
            
            # make sure there are no more items
            checkDefault(splitted, [])
            
    def generate(self):
        data = [
            self.localIP,
            self.ircServer,
            self.ircPort,
            self.welcomeMessage,
            self.version,
            self.toDo1,
            self.toDo2,
            self.toDo3,
            self.toDo4,
            self.ircChannel,
            'end'
        ]
        return urlbase64.encode(
        ','.join([str(item) for item in data])
        )
        

class Server(object):
    gameType = None
    name = None
    map = None
    currentPlayers = None
    maxPlayers = None
    host = None
    port = None
    version = '1.44'
    passworded = None
    dedicated = True
    def __init__(self, data = None, withGameType = False):
        if data:
            splitted = urlbase64.decode(data).split(',')
            
            checkDefault(splitted.pop(), 'end')
            
            if withGameType:
                self.gameType = splitted.pop(0)
            self.name = splitted.pop(0)
            self.map = splitted.pop(0)
            self.currentPlayers = int(splitted.pop(0))
            self.maxPlayers = int(splitted.pop(0))
            self.host = splitted.pop(0)
            self.port = int(splitted.pop(0))
            self.version = splitted.pop(0)
            passworded = int(splitted.pop(0))
            
            if passworded >= 2:
                passworded -= 2
                self.dedicated = True
            else:
                self.dedicated = False
                
            self.passworded = bool(passworded)

            checkDefault(splitted, [])
    
    def generate(self, withGameType = False):
        passworded = int(self.passworded)
        if self.dedicated:
            passworded += 2
        data = [
            self.name,
            self.map,
            self.currentPlayers,
            self.maxPlayers,
            self.host,
            self.port,
            self.version,
            passworded,
            'end'
        ]
        if withGameType:
            data.insert(0, self.gameType)

        return urlbase64.encode(
        ','.join([str(item) for item in data])
        )
        
class Servers(object):
    servers = None
    def __init__(self, data = None):
        servers = []
        if data:
            splitted = data.split('\n')
            for line in splitted:
                servers.append(Server(line, True))
        self.servers = servers