# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

# Copyright (c) 2009 Mathias Kaerlev.
# See LICENSE for details.

import urllib2

import urllib

import time

from urlparse import urlparse, urlunparse

from twisted.internet import reactor, defer

from twisted.web.client import getPage

from pySndo.network.site import parsers

from pySndo.locals import *

from pySndo.network.client import ClientInfoProtocol, ClientInfoFactory

from pySndo.game.weapon import Glock, SIGP, DesertEagle, Benelli, MP5, Galil, \
    SVD, AK47, Colt, G3A3, SR60, AW50
    
sndHeaders = {
    'Host' : 'www.seekanddread.de',
    'From' : 'webmaster@seekanddread.de'#,
    #'User-Agent' : 'HTTPTool/1.0'
}

sndAgent = 'HTTPTool/1.0'

SITE = 'http://www.seekanddread.de'
LOGIN = '%s/Game/login.php' % SITE
SERVER_LIST = '%s/Game/svr_get.php' % SITE
SERVER_ADD = '%s/Game/svr_add.php' % SITE
SERVER_DELETE = '%s/Game/svr_del.php' % SITE

def setUrlParameters(url, parameters = {}):
    parser = list(urlparse(url))
    parser[4] = urllib.urlencode(parameters)
    return urlunparse(parser)

class RetrievedServer(object):
    name = ''
    host = ''
    port = 0
    ping = 0
    mapName = ''
    mapCrc = 0
    welcome = ''
    serverVersion = '1.44'
    currentPlayers = 0
    maxPlayers = 0
    
    weapons = {
        DesertEagle : True,
        Benelli : True,
        MP5 : True,
        AK47 : True,
        SVD : True,
        Colt : True,
        G3A3 : True,
        AW50 : True,
        Galil : True,
        SR60 : True
    }
    
    password = None
    
    gameKey = None

class DetailProtocol(ClientInfoProtocol):
    state = False
    initialTime = None
    timeOut = None
    def connectionMade(self):
        ClientInfoProtocol.connectionMade(self)
        self.timeOut = reactor.callLater(self.factory.timeout, self.timedOut)
        
    def sendType(self, *arg, **kw):
        self.state += 1
        ClientInfoProtocol.sendType(self, *arg, **kw)
        if self.state == 2:
            self.initialTime = time.time()
        
    def timedOut(self):
        self.transport.loseConnection()
        
    def initialReceived(self, initial):
        ping = int((time.time() - self.initialTime) * 1000)
        self.factory.succeded = True
        self.timeOut.cancel()
        peer = self.transport.getPeer()
        self.factory.defer.callback((self.factory.name, peer.host, peer.port, ping, initial))
        self.transport.loseConnection()
        

class DetailFactory(ClientInfoFactory):
    protocol = DetailProtocol
    defer = None
    timeout = None
    name = None
    succeded = False

    def __init__(self, name, defer, timeout):
        self.name = name
        self.defer = defer
        self.timeout = timeout
        
    def clientConnectionFailed(self, connector, reason):
        self.defer.errback(reason)
        
    def clientConnectionLost(self, connector, reason):
        if not self.succeded:
            self.defer.errback(reason)

class Transmitter(object):
    _defer = None
    parser = None
    _defaultPage = None
    
    def __init__(self, url = None):
        self._defer = defer.Deferred()
        
        getPage(url or self._defaultPage, agent = sndAgent, headers = sndHeaders).addCallback(self._gotPage
            ).addErrback(self._defer.errback)
    
    def _gotPage(self, content):
        raise NotImplementedError
        
    def _setParser(self, parser):
        self.parser = parser
        self._defer.callback(parser)
        
    def getParser(self):
        return self._defer

class Login(Transmitter):
    _defaultPage = LOGIN
    
    def _gotPage(self, content):
        self._setParser(parsers.Login(content))
        
class Servers(Transmitter):
    _defaultPage = SERVER_LIST
    detailDefer = None
    def _gotPage(self, content):
        self._setParser(parsers.Servers(content))
        
    def getDetails(self, timeout = 5):
        defers = []
        factories = []
        for server in self.parser.servers:
            d = defer.Deferred()
            factories.append((DetailFactory(server.name, d, timeout), server.host, server.port+1))
            defers.append(d)
        deferList = defer.DeferredList(defers, consumeErrors = True).addCallback(self._gotServers).addErrback(self._timedOut)
        for (factory, host, port) in factories:
            reactor.connectTCP(host, port, factory, timeout = timeout)
        d = defer.Deferred()
        self.detailDefer = d
        return d
        
    def _gotServers(self, result):
        servers = []
        for (success, data) in result:
            if success:
                name, host, port, ping, initial = data
                newRetrieved = RetrievedServer()
                vars(newRetrieved).update(vars(initial))
                newRetrieved.ping = ping
                newRetrieved.host = host
                newRetrieved.port = port-1
                newRetrieved.name = name
                servers.append(newRetrieved)
        self.detailDefer.callback(servers)
    
    def _timedOut(self, reason):
        pass

class AddServer(Transmitter):
    _defaultPage = SERVER_ADD
    def __init__(self, serverObject, url = None):
        self._defer = defer.Deferred()
        
        url = url or self._defaultPage
        
        url = setUrlParameters(url, {'sndserver' : serverObject.generate()})
        
        getPage(url, agent = sndAgent, headers = sndHeaders).addCallback(self._gotPage
            ).addErrback(self._defer.errback)

    def _gotPage(self, content):
        self._setParser(None)
        
class DeleteServer(Transmitter):
    _defaultPage = SERVER_DELETE

    def _gotPage(self, content):
        self._setParser(None)